#include "StdAfx.h"
#include <setupapi.h>
#include "DeviceProf.h"
#include "misc.h"
#include "resource.h"

static const TCHAR *ALLOW_STRING = _T("ALLOW,");
static const size_t ALLOW_LENGTH = _tcslen( ALLOW_STRING );
static const TCHAR *DENY_STRING = _T("DENY,");
static const size_t DENY_LENGTH = _tcslen( DENY_STRING );

DeviceProfile::DeviceProfile(void)
	: PrivateProfile( _T(".ini") )
{
}

DeviceProfile::~DeviceProfile(void)
{
}

BOOL DeviceProfile::Load(void)
{
	FILE *fp;
	UINT c = 1;
	GUID guid;
	TCHAR tmpStr[1024];
	DWORD dwTmp;
	ENUM_CLASS ecTmp;
	BOOL bOldProfile = FALSE;
	tstring tmpApp;

	if ( _tfopen_s( &fp, profName, _T("rt") ) ) {
		MessageBoxWithString( hMainWnd, IDS_ERR_INIOPENFAILED, MB_OK | MB_ICONEXCLAMATION );
		return FALSE;
	}

	tmpApp = szTitle;
	tmpApp = _T("[") + tmpApp + _T("]\n");

	_fgetts( tmpStr, 1024, fp );
	if ( tmpApp == tmpStr ) {
		fclose( fp );
		bOldProfile = TRUE;
	}
	else {
		fseek( fp, 0, SEEK_SET );
	}

	do {
		TCHAR number[20];
		TCHAR tmpGuid[20];

		if ( bOldProfile ) {
			_stprintf_s( number, 20, _T("%d"), c++ );
			if ( !GetPrivateProfileString( szTitle, number, NULL, tmpStr, 1024, profName ) ) {
				break;
			}
		}
		else {
			_fgetts( tmpStr, 1024, fp );
			if ( feof( fp ) ) break;

			TCHAR *p = tmpStr + _tcslen(tmpStr);
			while( p-- >= tmpStr ) {
				if ( ( *p == _T('\r') ) || ( *p == _T('\n') ) ) {
					*p = 0;
				}
			}

			if ( ( tmpStr[0] == 0 ) || ( tmpStr[0] == _T(';') ) ) continue;
		}

		if ( _tcsnicmp( tmpStr, ALLOW_STRING, ALLOW_LENGTH ) == 0 ) {
			tmpApp = tmpStr + ALLOW_LENGTH;
			allowName.push_back( tmpApp );
			continue;
		}
		else if ( _tcsnicmp( tmpStr, DENY_STRING, DENY_LENGTH ) == 0 ) {
			tmpApp = tmpStr + DENY_LENGTH;
			denyName.push_back( tmpApp );
			continue;
		}
		else if ( ( _tcslen( tmpStr ) < 40 ) ||
			 ( tmpStr[ 0] != _T('{') ) ||
			 ( tmpStr[ 9] != _T('-') ) ||
			 ( tmpStr[14] != _T('-') ) ||
			 ( tmpStr[19] != _T('-') ) ||
			 ( tmpStr[24] != _T('-') ) ||
			 ( tmpStr[37] != _T('}') ) ||
			 ( tmpStr[38] != _T(',') ) ) {
			MessageBoxWithString( hMainWnd, IDS_ERR_INIFORMAT, MB_OK | MB_ICONEXCLAMATION );
			break;
		}

		tmpGuid[0] = 0;
		_tcsncpy_s( tmpGuid, 20, tmpStr + 1, 8 );
		if ( !_stscanf_s( tmpGuid, _T("%x"), &guid.Data1 ) ) break;

		_tcsncpy_s( tmpGuid, 20, tmpStr + 10, 4 );
		if ( !_stscanf_s( tmpGuid, _T("%x"), &dwTmp ) ) break;
		guid.Data2 = (WORD)dwTmp;

		_tcsncpy_s( tmpGuid, 20, tmpStr + 15, 4 );
		if ( !_stscanf_s( tmpGuid, _T("%x"), &dwTmp ) ) break;
		guid.Data3 = (WORD)dwTmp;

		_tcsncpy_s( tmpGuid, 20, tmpStr + 20, 4 );
		if ( !_stscanf_s( tmpGuid, _T("%x"), &dwTmp ) ) break;
		guid.Data4[0] = (BYTE)(dwTmp >> 8);
		guid.Data4[1] = dwTmp & 0xff;

		for ( int c = 0; c < 6; c++ ) {
			_tcsncpy_s( tmpGuid, 20, tmpStr + 25 + c * 2, 2 );
			if ( !_stscanf_s( tmpGuid, _T("%x"), &dwTmp ) ) break;
			guid.Data4[c+2] = (BYTE)dwTmp;
		}

		ecTmp.guid = guid;
		ecTmp.name = tmpStr+39;

		if ( ecTmp.name.find( _T(",") ) != std::string::npos ) {
			ecTmp.name.erase( ecTmp.name.find( _T(",") ) );
			ecTmp.flag = DIGCF_DEVICEINTERFACE;
		}
		else {
			ecTmp.flag = 0;
		}

		enumClass.push_back( ecTmp );

	} while(1);

	if ( !bOldProfile ) fclose( fp );

	if ( enumClass.size() == 0 ) {
		MessageBoxWithString( hMainWnd, IDS_ERR_INIFORMAT, MB_OK | MB_ICONEXCLAMATION );
		return FALSE;
	}

	return TRUE;
}

size_t DeviceProfile::size(void)
{
	return enumClass.size();
}
