#include "StdAfx.h"
#include "MainDialog.h"
#include "resource.h"
#include "misc.h"
#include <windowsx.h>

#define IDM_SYS_ABOUT 0x10

MainDialog::MainDialog( UINT idResource )
	: ReposDialog( idResource )
	, imageList( NULL )
	, curSel( 0 )
	, numSelected( 0 )
	, m_aboutDlg( NULL )
{
	static REPOS_ITEM items[] = {
		{ IDC_LIST_LOG,			ANCHOR_LEFT | ANCHOR_TOP | ANCHOR_RIGHT | ANCHOR_BOTTOM },
		{ IDC_CMB_CLASS,		ANCHOR_LEFT | ANCHOR_BOTTOM },
		{ IDC_BTN_SEARCH,		ANCHOR_LEFT | ANCHOR_BOTTOM },
		{ IDC_BTN_REMOVE,		ANCHOR_LEFT | ANCHOR_BOTTOM },
		{ IDOK,					ANCHOR_RIGHT | ANCHOR_BOTTOM },
		{ IDCANCEL,				ANCHOR_RIGHT | ANCHOR_BOTTOM },
		{ IDC_BTN_SELREV,		ANCHOR_LEFT | ANCHOR_BOTTOM },
		{ IDC_BTN_SEARCHALL,	ANCHOR_LEFT | ANCHOR_BOTTOM },
	};

	reposItems = items;
	reposNum = _countof(items);
	memset( sortMethod, 0, sizeof(sortMethod) );
}

MainDialog::~MainDialog(void)
{
	UnregisterClass( szTitle, hInst );
}

INT_PTR CALLBACK MainDialog::DlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	ReposDialog::ReposProc( uMsg, wParam, lParam );

	switch( uMsg )
	{
	HANDLE_MSG( m_hDlg, WM_INITDIALOG,	Cls_OnInitDialog );
	HANDLE_MSG( m_hDlg, WM_SYSCOMMAND,	Cls_OnSysCommand );
	HANDLE_MSG( m_hDlg, WM_COMMAND,		Cls_OnCommand );
	HANDLE_MSG( m_hDlg, WM_DESTROY,		Cls_OnDestroy );

	case WM_DEVMAN_ENUMEND:	Cls_OnDevManEnumEnd();										break;
	case WM_DEVMAN_LISTDEL:	ListView_DeleteItem( hList, wParam );						break;
	case WM_DEVMAN_DEVCHG:	Cls_OnDevManDevChg( (int)wParam, (REMOVE_STATE)lParam );	break;
	case WM_DEVMAN_DELEND:	Cls_OnDevManDelEnd();										break;
	case WM_GETMINMAXINFO:	MainSetMin( &((MINMAXINFO*)lParam)->ptMinTrackSize );		break;
	case WM_NOTIFY:			return Cls_OnNotify( wParam, lParam );						break;

	default:
		return DefWindowProc(m_hDlg, uMsg, wParam, lParam);
		break;
	}

	return 0;
}

BOOL MainDialog::InitInstance( HWND hParent )
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)_RootWndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= DLGWINDOWEXTRA;
	wcex.hInstance		= hInst;
	wcex.hIcon			= LoadIcon(hInst, MAKEINTRESOURCE(IDI_DIALOG));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_BTNFACE+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szTitle;
	wcex.hIconSm		= NULL;

	return RegisterClassEx(&wcex);
}

BOOL MainDialog::DoModeless(HWND hParent, int nCmdShow)
{
	BOOL bRet = FALSE;

	if ( InitInstance() ) {
		if ( bRet = Dialog::DoModeless( hParent, nCmdShow ) ) {
			SetWindowText( m_hDlg, szTitle );
		}
	}
	return bRet;
}

void MainDialog::InsertSystemMenu( void )
{
	MENUITEMINFO mii;
	HMENU hSysMenu;
	TCHAR szTemp[256];
	int nMenuItemCount;

	memset ( &mii, 0, sizeof(MENUITEMINFO) );
	mii.cbSize = sizeof(MENUITEMINFO);

	mii.fMask = MIIM_TYPE;
	mii.fType = MFT_SEPARATOR;

	// Zp[^ǉ
	hSysMenu = GetSystemMenu( m_hDlg, FALSE );
	nMenuItemCount = GetMenuItemCount ( hSysMenu );
	InsertMenuItem ( hSysMenu, nMenuItemCount -1, TRUE, &mii );

	LoadString( hInst, IDS_ABOUTBOX, szTemp, 256 );
	mii.fMask	   = MIIM_ID | MIIM_TYPE;
	mii.fType	   = MFT_STRING;
	mii.wID 	   = IDM_SYS_ABOUT;
	mii.dwTypeData = szTemp;
	mii.cch 	   = (UINT)_tcslen(szTemp);

	InsertMenuItem( hSysMenu, nMenuItemCount -1, TRUE, &mii );
	DrawMenuBar( m_hDlg );
}

void MainDialog::CalcMinSize(void)
{
	RECT rect1, rectDlg;

	GetClientRect( m_hDlg, &prevWinRect );
	GetWindowRect( m_hDlg, &rectDlg );

	GetDlgItemRect( IDC_LIST_LOG, &rect1 );
	ptMinSize.x = rectDlg.right - rectDlg.left;
	ptMinSize.y = rectDlg.bottom - rectDlg.top - ( rect1.bottom - rect1.top );

	DWORD lvSize = ListView_ApproximateViewRect( hList, -1, -1, 1 );
	ptMinSize.y += HIWORD(lvSize);
}

int MainDialog::InitControls(void)
{
	LVCOLUMN	lvc;
	int 		i;
	const UINT	captionTable[_NUMBER_OF_COLUMN] = { 0, IDS_COLUMN_PATH, IDS_COLUMN_NAME };
	int			width[_NUMBER_OF_COLUMN] = { 40, 100, 100 };
	int 		err = 0;
	HICON		hIconAlert = NULL, hIconDelete = NULL;

	hIconAlert  = (HICON)LoadIcon( hInst, MAKEINTRESOURCE( IDI_ALERT ) );
	hIconDelete = (HICON)LoadIcon( hInst, MAKEINTRESOURCE( IDI_DELETE ) );

	imageList = ImageList_Create( 16, 16, ILC_COLOR8 | ILC_MASK, 4, 4 );
	ImageList_AddIcon( imageList, hIconAlert );
	ImageList_AddIcon( imageList, hIconDelete );
	ListView_SetImageList( hList, imageList, LVSIL_SMALL );

	lvc.mask = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;	 // LtO
	for (i = 0; i < _NUMBER_OF_COLUMN; i++)
	{
		TCHAR szCaption[256];
		if ( captionTable[i] != 0 ) {
			LoadString( hInst, captionTable[i], szCaption, 256 );
		}
		else {
			szCaption[0] = 0;
		}

		lvc.iSubItem	= i;					// TuACeԍ
		lvc.pszText 	= szCaption;			// oeLXg
		lvc.cx			= width[i];				// 
		if ( ListView_InsertColumn( hList, i, &lvc ) == -1 ) {
			err = 1;
			break;
		}
	}

	DWORD dwStyle = ListView_GetExtendedListViewStyle( hList );
	dwStyle |= LVS_EX_SUBITEMIMAGES | LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES;
	ListView_SetExtendedListViewStyle( hList, dwStyle );

	for ( UINT c = 0; c < deviceProf.size(); c++ ) {
		ComboBox_AddString( GetDlgItem( m_hDlg, IDC_CMB_CLASS ), deviceProf[c].name.c_str() );
	}
	ComboBox_SetCurSel( GetDlgItem( m_hDlg, IDC_CMB_CLASS ), 0 );
	if ( deviceProf.size() == 1 ) {
		EnableWindow( GetDlgItem( m_hDlg, IDC_CMB_CLASS ), FALSE );
	}

	AdjustComboBoxHeight( IDC_CMB_CLASS, (UINT)deviceProf.size() );

	return err;
}

int MainDialog::ListInsertItem( DISPLAY_LOG newline )
{
	LVITEM		lvi;
	int 		index = 0;
	int 		err = 0;
	int			num;

	num = ListView_GetItemCount( hList );

	// ACRƔԍ
	lvi.mask = LVIF_IMAGE | LVIF_PARAM;
	lvi.iItem = num;
	lvi.iSubItem = 0;
	lvi.lParam = num;
	lvi.iImage = GetImageIndex( newline.state );
	if ((index = ListView_InsertItem( hList, &lvi)) == -1) err = 1;

	// V{bNpX
	if (!err)
	{
		lvi.mask = LVIF_TEXT;
		lvi.iItem = num;
		lvi.iSubItem = 1;
		lvi.pszText = (LPTSTR)newline.path.c_str();
		if ( !ListView_SetItem( hList, &lvi ) ) err = 1;
	}
	// O
	if (!err)
	{
		lvi.iItem = num;
		lvi.iSubItem = 2;
		lvi.pszText = (LPTSTR)newline.name.c_str();
		if ( !ListView_SetItem( hList, &lvi ) ) err = 1;
	}

	return err;
}

void MainDialog::UpdateList( void )
{
	ListView_DeleteAllItems( hList );

	for ( int c = 0; c < (int)devMan.size(); c++ ) {
		ListInsertItem( devMan[c] );
	}
	for ( int c = 1; c < 3; c++ ) {
		ListView_SetColumnWidth( hList, c, LVSCW_AUTOSIZE_USEHEADER );
	}
}

int MainDialog::GetImageIndex( REMOVE_STATE state )
{
	int idx;

	switch( state )
	{
	case STATE_FAILED:
		idx = 0;
		break;
	case STATE_REMOVED:
		idx = 1;
		break;
	default:
		idx = -1;
	}

	return idx;
}

void MainDialog::SetButtonState( UI_STATE state )
{
	switch( state )
	{
	case STATE_INITIAL:
		EnableItem( IDC_BTN_SEARCH, TRUE );
		EnableItem( IDC_BTN_SEARCHALL, TRUE );
		EnableItem( IDC_BTN_REMOVE, FALSE );
		EnableItem( IDC_BTN_SELREV, FALSE );
		EnableItem( IDC_CMB_CLASS, TRUE );
		::SetFocus( GetDlgItem( m_hDlg, IDC_CMB_CLASS ) );
		break;

	case STATE_INSEARCH:
	case STATE_INDELETE:
		EnableItem( IDC_BTN_SEARCH, FALSE );
		EnableItem( IDC_BTN_SEARCHALL, FALSE );
		EnableItem( IDC_BTN_REMOVE, FALSE );
		EnableItem( IDC_BTN_SELREV, FALSE );
		EnableItem( IDC_CMB_CLASS, FALSE );
		break;

	case STATE_DEVEXIST:
		EnableItem( IDC_BTN_SEARCH, TRUE );
		EnableItem( IDC_BTN_SEARCHALL, TRUE );
		EnableItem( IDC_BTN_REMOVE, TRUE );
		EnableItem( IDC_BTN_SELREV, TRUE );
		EnableItem( IDC_CMB_CLASS, TRUE );
		::SetFocus( hList );
		break;
	}
}

void MainDialog::MainSetMin( POINT *pt )
{
	int height;
	LONG_PTR listStyle;

	listStyle = GetWindowLongPtr( hList, GWL_STYLE );
	if ( listStyle & WS_HSCROLL ) {
		height = GetSystemMetrics( SM_CYHSCROLL );
		pt->y += height;
	}
}

void MainDialog::Cls_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch( id )
	{
	case IDC_BTN_SEARCH:
		ClearSortMethod();
		ListView_DeleteAllItems( hList );
		numSelected = 0;
		curSel = ComboBox_GetCurSel( GetDlgItem( m_hDlg, IDC_CMB_CLASS ) );
		if ( devMan.Search( deviceProf[curSel], m_hDlg ) ) {
			SetButtonState( STATE_INSEARCH );
		}
		break;

	case IDC_BTN_SEARCHALL:
		ClearSortMethod();
		ListView_DeleteAllItems( hList );
		numSelected = deviceProf.size();
		curSel = 0;
		ComboBox_SetCurSel( GetDlgItem( m_hDlg, IDC_CMB_CLASS ), 0 );
		if ( devMan.Search( deviceProf[curSel], m_hDlg ) ) {
			SetButtonState( STATE_INSEARCH );
		}
		break;

	case IDC_BTN_REMOVE:
		numSelected = 0;
		for ( UINT c = 0; c < devMan.size(); c++ ) {
			BOOL bSelected = ListView_GetCheckState( hList, c );
			if ( bSelected ) numSelected++;
			devMan.SetSelect( c, bSelected );
		}
		if ( numSelected == 0 ) {
			MessageBoxWithString( m_hDlg, IDS_NOSELECT, MB_OK );
		}
		else {
			if ( ( !bOptAuto ) && ( MessageBoxWithString( m_hDlg, IDS_Q_DELETE, MB_YESNO | MB_ICONQUESTION ) == IDNO ) ) {
				break;
			}
			if ( devMan.Delete( m_hDlg ) ) {
				SetButtonState( STATE_INDELETE );
			}
		}
		break;

	case IDC_BTN_SELREV:
		for ( UINT c = 0; c < devMan.size(); c++ ) {
			BOOL bSelected = ~ListView_GetCheckState( hList, c ) & 1;
			ListView_SetCheckState( hList, c, bSelected );
			devMan.SetSelect( c, bSelected );
		}
		break;

	case IDOK:
	case IDCANCEL:
		DestroyWindow( m_hDlg );
		break;
	}
}

BOOL MainDialog::Cls_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	hMainWnd = m_hDlg;
	hList = GetDlgItem( m_hDlg, IDC_LIST_LOG );
	InsertSystemMenu();
	deviceProf.Load();
	devMan.SetAllowList( deviceProf.allowList() );
	devMan.SetDenyList( deviceProf.denyList() );
	InitControls();
	CalcMinSize();
	SetButtonState( STATE_INITIAL );
	if ( bOptAuto ) {
		PostMessage( m_hDlg, WM_COMMAND, MAKEWPARAM( IDC_BTN_SEARCHALL, 0 ), 0 );
	}

	return 0;
}

void MainDialog::Cls_OnSysCommand(HWND hwnd, UINT cmd, int x, int y)
{
	switch( cmd & 0xfff0 )
	{
	case IDM_SYS_ABOUT:
		if ( m_aboutDlg ) {
			m_aboutDlg->SetFocus();
		}
		else {
			m_aboutDlg = new AboutDlg(IDD_ABOUTBOX);
			m_aboutDlg->DoModal(m_hDlg);
			delete m_aboutDlg;
			m_aboutDlg = NULL;
		}
		break;
	default:
		DefWindowProc(hwnd, WM_SYSCOMMAND, cmd, MAKELPARAM( x, y ) );
	}
}

void MainDialog::Cls_OnDevManEnumEnd( void )
{
	devMan.Join();
	curSel++;
	if ( curSel < (int)numSelected ) {
		ComboBox_SetCurSel( GetDlgItem( m_hDlg, IDC_CMB_CLASS ), curSel );
		devMan.Search( deviceProf[curSel], m_hDlg, TRUE );
	}
	else {
		if ( numSelected > 0 ) {
			ComboBox_SetCurSel( GetDlgItem( m_hDlg, IDC_CMB_CLASS ), 0 );
			numSelected = 0;
		}
		UpdateList();
		SetButtonState( devMan.size() > 0 ? STATE_DEVEXIST : STATE_INITIAL );
		if ( bOptAuto && bOptDelete ) {
			PostMessage( m_hDlg, WM_COMMAND, MAKEWPARAM( IDC_BTN_SELREV, 0 ), 0 );
			PostMessage( m_hDlg, WM_COMMAND, MAKEWPARAM( IDC_BTN_REMOVE, 0 ), 0 );
		}
	}
}

void MainDialog::Cls_OnDevManDelEnd( void )
{
	devMan.Join();
	SetButtonState( devMan.size() > 0 ? STATE_DEVEXIST : STATE_INITIAL );
	if ( bOptAuto && bOptExit ) {
		PostMessage( m_hDlg, WM_COMMAND, MAKEWPARAM( IDCANCEL, 0 ), 0 );
	}
}

void MainDialog::Cls_OnDevManDevChg( int idx, REMOVE_STATE state )
{
	LVITEM li;

	li.mask = LVIF_IMAGE;
	li.iSubItem = 0;
	li.iItem = idx;
	li.iImage = GetImageIndex( state );
	ListView_SetItem( hList, &li );
}

void MainDialog::Cls_OnDestroy(HWND hwnd)
{
	ImageList_Destroy( imageList );
	PostQuitMessage( 0 );
}

BOOL MainDialog::Cls_OnNotify( WPARAM wParam, LPARAM lParam )
{
	NMHDR* hdr = (NMHDR*)lParam;
	LPNMLISTVIEW pnmlist;
	static int num = 0;

	if ( hdr->hwndFrom == hList ) {
		switch( hdr->code )
		{
		case LVN_ITEMCHANGED:
			pnmlist = (LPNMLISTVIEW)lParam;
			if ( ( pnmlist->uNewState & LVIS_STATEIMAGEMASK ) == 0 ) {

				SHORT ctrlState = GetKeyState( VK_CONTROL );
				if ( ListView_GetItemState( hList, pnmlist->iItem, LVIS_SELECTED ) ) {
					if ( ( ctrlState >= 0 ) ||
						( ( ctrlState < 0 ) && ( !ListView_GetItemState( hList, pnmlist->iItem, LVIS_FOCUSED ) ) ) ) {
						ListView_SetCheckState( hList, pnmlist->iItem, TRUE );
					}
				}
				else {
					if ( ( ctrlState >= 0 ) ||
						( ( ctrlState < 0 ) && ( !ListView_GetItemState( hList, pnmlist->iItem, LVIS_FOCUSED ) ) ) ) {
						ListView_SetCheckState( hList, pnmlist->iItem, FALSE );
					}
				}
			}
			else {
				if ( ( pnmlist->uNewState & LVIS_STATEIMAGEMASK ) == INDEXTOSTATEIMAGEMASK(2) ) {
					ListView_SetItemState( hList, pnmlist->iItem, LVIS_SELECTED, LVIS_SELECTED );
				}
				else {
					ListView_SetItemState( hList, pnmlist->iItem, 0, LVIS_SELECTED );
				}
			}
			break;

		case LVN_COLUMNCLICK:
			pnmlist = (LPNMLISTVIEW)lParam;
			if ( sortMethod[ pnmlist->iSubItem ] == SORT_UP ) {
				sortMethod[ pnmlist->iSubItem ] = SORT_DOWN;
			}
			else {
				sortMethod[ pnmlist->iSubItem ] = SORT_UP;
			}
			UpdateHeaderSortIcon();

			SORT_CONTEXT context = { hList, pnmlist->iSubItem, sortMethod[ pnmlist->iSubItem ] };
			ListView_SortItems( hList, SortCompareProc, (LPARAM)&context );

			std::vector <tstring> newPathList;
			tstring tmp;
			int num = ListView_GetItemCount( hList );
			if ( num ) {
				for ( int c = 0; c < num; c++ ) {
					GetListViewItemText( hList, c, 1, tmp );
					newPathList.push_back( tmp );
				}
				if ( !devMan.ApplySortedList( newPathList ) ) {
					MessageBoxWithString( m_hDlg, IDS_ERR_UPDATEDEVLIST, MB_OK | MB_ICONEXCLAMATION );
					PostMessage( m_hDlg, WM_COMMAND, MAKEWPARAM( IDCANCEL, 0 ), 0 );
				}
			}
			break;
		}
	}

	return FALSE;
}

int CALLBACK MainDialog::SortCompareProc( LPARAM lp1, LPARAM lp2, LPARAM lp3 )
{
	LVFINDINFO lvf;
	int nItem1, nItem2;
	tstring buf1, buf2;
	SORT_CONTEXT *context = (SORT_CONTEXT*)lp3;

	lvf.flags = LVFI_PARAM;
	lvf.lParam = lp1;
	nItem1 = ListView_FindItem( context->hList, -1, &lvf );

	lvf.lParam = lp2;
	nItem2 = ListView_FindItem( context->hList, -1, &lvf );

	if ( (int)context->sortSubItem == 0 ) {

		LVITEM li;
		int image1, image2;

		li.mask = LVIF_IMAGE;
		li.iSubItem = 0;
		li.iItem = nItem1;
		ListView_GetItem( context->hList, &li );
		image1 = li.iImage;

		li.iItem = nItem2;
		ListView_GetItem( context->hList, &li );
		image2 = li.iImage;

		if ( image1 == image2 ) return 0;
		else if ( context->sortMethod == SORT_UP ) {
			return ( image1 > image2 ) ? 1 : -1;
		}
		else {
			return ( image1 < image2 ) ? 1 : -1;
		}
	}
	else {
		GetListViewItemText( context->hList, nItem1, (int)context->sortSubItem, buf1 );
		GetListViewItemText( context->hList, nItem2, (int)context->sortSubItem, buf2 );

		if ( context->sortMethod == SORT_UP ) {
			return buf1.compare( buf2 );
		}
		else {
			return buf2.compare( buf1 );
		}
	}
}

void MainDialog::UpdateHeaderSortIcon( void )
{
	HDITEM hi;
	HWND hHeader = ListView_GetHeader( hList );

	for ( int c = 0; c < _NUMBER_OF_COLUMN; c++ ) {

		hi.mask = HDI_FORMAT;
		Header_GetItem( hHeader, c, &hi );

		hi.fmt &= ~( HDF_SORTUP | HDF_SORTDOWN );
		if ( sortMethod[c] != SORT_NONE ) {
			hi.fmt |= ( sortMethod[c] == SORT_UP ) ? HDF_SORTUP : HDF_SORTDOWN;
		}

		Header_SetItem( hHeader, c, &hi );
	}
}

void MainDialog::ClearSortMethod( void )
{
	for ( int c = 0; c < _NUMBER_OF_COLUMN; c++ ) {
		sortMethod[c] = SORT_NONE;
	}
	UpdateHeaderSortIcon();
}

void MainDialog::GetListViewItemText( HWND hWndList, int idx, int sub, tstring &str )
{
	LVITEM li;
	int n = 0, len;
	TCHAR *dummy = NULL;

	li.mask = LVIF_TEXT;
	li.iItem = idx;
	li.iSubItem = sub;

	do {
		if ( dummy ) {
			delete[] dummy;
		}
		n += 1024;
		dummy = new TCHAR[n];

		li.pszText = dummy;
		li.cchTextMax = n;
		len = (int)SendMessage( hWndList, LVM_GETITEMTEXT, idx, (LPARAM)&li );
	} while( n == ( len + 1 ) );

	str = dummy;
	delete[] dummy;
}
